@@ -2,17 +2,20 @@
 Functions for setting up testing SQL tables.
 """
 
+import sqlalchemy as _sa
+import sqlalchemy.engine as _sa_engine
+
 from fullmetalalchemy.create import create_table
 from fullmetalalchemy.insert import insert_records
 
 
-def setup(engine):
+def setup(engine: _sa_engine.Engine) -> _sa.Table:
     table = create_test_table(engine)
     insert_test_records(table, engine)
     return table
 
 
-def create_test_table(engine):
+def create_test_table(engine: _sa_engine.Engine) -> _sa.Table:
     return create_table(table_name='xy',
                         column_names=['id', 'x', 'y'],
                         column_types=[int, int, int],
@@ -21,7 +24,7 @@ def create_test_table(engine):
                         if_exists='replace')
 
 
-def create_second_test_table(engine):
+def create_second_test_table(engine: _sa_engine.Engine) -> _sa.Table:
     return create_table(table_name='xyz',
                         column_names=['id', 'x', 'y', 'z'],
                         column_types=[int, int, int, int],
@@ -30,7 +33,7 @@ def create_second_test_table(engine):
                         if_exists='replace')
 
 
-def insert_test_records(table, engine):
+def insert_test_records(table: _sa.Table, engine: _sa_engine.Engine) -> None:
     records = [
             {'id': 1, 'x': 1, 'y': 2},
             {'id': 2, 'x': 2, 'y': 4},
@@ -38,5 +41,4 @@ def insert_test_records(table, engine):
             {'id': 4, 'x': 8, 'y': 11}
             ]
     insert_records(table, records, engine)
-    
-    
\ No newline at end of file
+
